;##################################################################
; JetStream
;##################################################################

#include "JetStream.h"
#include "odbc.asm"

IDD_DIALOG	equ		1000
IMG_TABLETYPES equ	9500
IMG_DATATYPES	equ 9000
IMG_NUMERIC equ		1
IMG_STRING	equ		0
IMG_BOOL	equ		1

CONST SECTION
	ColumnInfo1	DB "Description",0
	ColumnInfo2	DB "Value",0

	ValueLabel01 DB "Column base name",0 ; SQL_DESC_BASE_COLUMN_NAME
	ValueLabel02 DB "Column Name",0 ; SQL_DESC_NAME
	ValueLabel03 DB "Column Label",0 ; SQL_DESC_LABEL
	ValueLabel04 DB "Column length",0 ; SQL_DESC_LENGTH
	ValueLabel05 DB "Display size",0 ; SQL_DESC_DISPLAY_SIZE
	ValueLabel06 DB "Data type",0 ; SQL_DESC_TYPE_NAME
	ValueLabel07 DB "Data precision",0 ; SQL_DESC_PRECISION
	ValueLabel08 DB "Autoincrement",0 ; SQL_DESC_AUTO_UNIQUE_VALUE
	ValueLabel09 DB "NULL allowed",0 ; SQL_DESC_NULLABLE
	ValueLabel10 DB "Case sensitive",0 ; SQL_DESC_CASE_SENSITIVE
	ValueLabel11 DB "Unsigned",0 ; SQL_DESC_UNSIGNED
	ValueLabel12 DB "Updatable",0 ; SQL_DESC_UPDATABLE

	szError DB "Error",0

DATA SECTION
	hInstance		DD	?
	hHeap			DD	?
	hImageList		DD	?
	hImlTreeView	DD	?
	hTreeview		DD	?
	hInfoListview	DD	?
	hCurrentRoot	DD	?
	hImlToolbar		DD	?
	fIsLoaded		DD	?

CODE SECTION

Start:
	invoke GetModuleHandle, 0
	mov [hInstance],eax
	invoke HeapCreate,NULL,0100000h,0100000h
	mov [hHeap], eax
	invoke InitCommonControls
	invoke ImageList_LoadImage,[hInstance], IMG_DATATYPES,0,0,CLR_NONE, \
		IMAGE_BITMAP,LR_LOADTRANSPARENT
	mov [hImageList],eax

	invoke ImageList_LoadImage,[hInstance], IMG_TABLETYPES,0,0,CLR_NONE, \
		IMAGE_BITMAP,LR_LOADTRANSPARENT
	mov [hImlTreeView],eax

	invoke DialogBoxParam,[hInstance], IDD_DIALOG, 0, ADDR DlgProc, 0

	invoke ImageList_Destroy,[hImageList]
	invoke ImageList_Destroy,[hImlTreeView]
	invoke HeapDestroy, [hHeap]
	invoke ExitProcess,0

DlgProc FRAME hwnd,uMsg,wParam,lParam
	uses ebx,edi,esi

	mov eax,[uMsg]
	cmp eax,WM_INITDIALOG
	jne >>.WM_COMMAND
		invoke GetDlgItem,[hwnd],1001
		mov [hInfoListview],eax
		invoke GetDlgItem,[hwnd],1002
		mov [hTreeview], eax
		invoke SetupListview,[hInfoListview]
		
		invoke SendMessage,[hTreeview],TVM_SETIMAGELIST,0,[hImlTreeView]
		mov eax, TRUE
		ret

	.WM_COMMAND
	cmp eax,WM_COMMAND
	jne >>.WM_NOTIFY
		movzx eax,W[wParam+2]
		movzx edx,W[wParam]
		cmp edx,10002
		jne >
			cmp D[fIsLoaded], TRUE
			je >>.EXIT
			invoke OpenDatabase,NULL,offset szDbFile
			test eax,eax
			js >>.EXIT
			invoke ODBCConnect, offset szDbFile
			test eax,eax
			js >>.EXIT
			invoke ODBCGetTableList,offset szDbFile
			call ODBCDisconnect
			mov D[fIsLoaded], TRUE
		:
		cmp edx,10003
		jne >>
			cmp D[fIsLoaded], FALSE
			je >>.EXIT
			invoke SendMessage, [hInfoListview], LVM_DELETEALLITEMS, 0, 0
			invoke SendMessage, [hTreeview], TVM_EXPAND,TVE_COLLAPSERESET | TVE_COLLAPSE, 0
			invoke SendMessage,[hTreeview], TVM_DELETEITEM, 0, 0
			invoke HeapDestroy,[hHeap]
			invoke HeapCreate,NULL,0100000h,0100000h
			mov [hHeap], eax
			mov D[fIsLoaded], FALSE
		:
		cmp edx,10004
		jne >
			invoke PostMessage,[hwnd],WM_CLOSE,0,0
		:
		mov eax, TRUE
		ret

	.WM_NOTIFY
	cmp eax,WM_NOTIFY
	jne >>.WM_CLOSE
		mov edi,[lParam]
		cmp D[edi+NMHDR.code],TVN_SELCHANGED
		jne >>.WM_CLOSE
			// Root items have 0 as their lParam, child items have the parent handle
			mov eax,[edi+NMTREEVIEW.itemNew.lParam]
			mov eax,[eax+COLUMNINFOSTRUCT.hParent]
			test eax,eax
			jz >.EXIT
				// This is a child item so display the parameter information
				invoke BuildColumnInfoListview,[hTreeview],[hInfoListview],[edi+NMTREEVIEW.itemNew.hItem]
				mov eax,0
				ret

	.WM_CLOSE
	cmp eax,WM_CLOSE
	jne >.DEFPROC
		invoke EndDialog,[hwnd],0

	.DEFPROC
		mov eax,FALSE
		ret

	.EXIT

	mov eax, TRUE
	ret
ENDF

SetupListview FRAME hLv
	LOCAL lvc			:LVCOLUMN

	// Set up the listview
	mov D[lvc.mask],LVCF_TEXT + LVCF_WIDTH
	mov [lvc.pszText],offset ColumnInfo1
	mov D[lvc.cx], 150
	invoke SendMessage,[hLv],LVM_INSERTCOLUMN,0,offset lvc

	mov D[lvc.mask],LVCF_TEXT + LVCF_WIDTH
	mov [lvc.pszText],offset ColumnInfo2
	mov D[lvc.cx], 100
	invoke SendMessage,[hLv],LVM_INSERTCOLUMN,1,offset lvc

	invoke SendMessage,[hLv],LVM_SETEXTENDEDLISTVIEWSTYLE,LVS_EX_FULLROWSELECT,LVS_EX_FULLROWSELECT
	invoke SendMessage, [hLv], LVM_SETIMAGELIST,LVSIL_SMALL,[hImageList]
	
	RET
ENDF

BuildColumnInfoListview FRAME hTv, hLv, hItem
	LOCAL szTable[1024]	:B
	LOCAL dwColumn		:D
	LOCAL tvi			:TVITEM
	LOCAL lvc			:LVCOLUMN
	LOCAL retData[1024]	:B

	/*
	Information to be displayed

	Column base name - SQL_DESC_BASE_COLUMN_NAME
	Column Name - SQL_DESC_NAME
	Column Label - SQL_DESC_LABEL
	Column length - SQL_DESC_LENGTH
	Display size - SQL_DESC_DISPLAY_SIZE
	Data type - SQL_DESC_TYPE_NAME
	Data precision - SQL_DESC_PRECISION
	Autoincrement - SQL_DESC_AUTO_UNIQUE_VALUE
	NULL allowed - SQL_DESC_NULLABLE
	Case sensitive - SQL_DESC_CASE_SENSITIVE
	unsigned - SQL_DESC_UNSIGNED
	Updatable - SQL_DESC_UPDATABLE
	*/

	// Clear the listview of data
	invoke SendMessage, [hLv],LVM_DELETEALLITEMS,0,0

	// Get the currently selected column number and table name
	mov eax,[hItem]
	mov [tvi.hItem],eax
	mov D[tvi.mask], TVIF_PARAM
	invoke SendMessage,[hTv],TVM_GETITEM,0,offset tvi

	mov eax,[tvi.lParam]
	mov edx,[eax+COLUMNINFOSTRUCT.iColumn]
	mov [dwColumn],edx

	// Get the table name
	mov edx,[eax+COLUMNINFOSTRUCT.hParent]
	mov [tvi.hItem],edx
	mov D[tvi.mask], TVIF_TEXT
	lea eax,szTable
	mov [tvi.pszText], eax
	mov D[tvi.cchTextMax],1024
	invoke SendMessage,[hTv],TVM_GETITEM,0,offset tvi

	// Add the descriptions and values

	invoke ODBCConnect, offset szDbFile

	// SQL_DESC_BASE_COLUMN_NAME
	// String value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_BASE_COLUMN_NAME,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
	:
	invoke ListviewAddItem, [hLv], IMG_STRING, offset ValueLabel01, offset retData, 0
	
	// SQL_DESC_NAME
	// String value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_NAME,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
	:
	invoke ListviewAddItem, [hLv], IMG_STRING, offset ValueLabel02, offset retData, 0

	// SQL_DESC_LABEL
	// String value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_LABEL,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
	:
	invoke ListviewAddItem, [hLv], IMG_STRING, offset ValueLabel03, offset retData, 0

	// SQL_DESC_LENGTH
	// Numeric value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_LENGTH,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
	:
	invoke ListviewAddItem, [hLv], IMG_NUMERIC, offset ValueLabel04, offset retData, 0

	// SQL_DESC_DISPLAY_SIZE
	// Numeric value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_DISPLAY_SIZE,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
	:
	invoke ListviewAddItem, [hLv], IMG_NUMERIC, offset ValueLabel05, offset retData, 0

	// SQL_DESC_TYPE_NAME
	// String value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_TYPE_NAME,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
	:
	invoke ListviewAddItem, [hLv], IMG_STRING, offset ValueLabel06, offset retData, 0

	// SQL_DESC_PRECISION
	// Numeric value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_PRECISION,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
	:
	invoke ListviewAddItem, [hLv], IMG_NUMERIC, offset ValueLabel07, offset retData, 0

	// SQL_DESC_AUTO_UNIQUE_VALUE
	// Boolean value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_AUTO_UNIQUE_VALUE,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
		jmp >.SQL_DESC_AUTO_UNIQUE_VALUE
	:
	invoke STRING2BOOL,offset retData
	.SQL_DESC_AUTO_UNIQUE_VALUE
	invoke ListviewAddItem, [hLv], IMG_NUMERIC, offset ValueLabel08, offset retData, 0

	// SQL_DESC_NULLABLE
	// Boolean value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_NULLABLE,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
		jmp >.SQL_DESC_NULLABLE
	:
	invoke STRING2BOOL,offset retData
	.SQL_DESC_NULLABLE
	invoke ListviewAddItem, [hLv], IMG_NUMERIC, offset ValueLabel09, offset retData, 0

	// SQL_DESC_CASE_SENSITIVE
	// Boolean value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_CASE_SENSITIVE,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
		jmp >.SQL_DESC_CASE_SENSITIVE
	:
	invoke STRING2BOOL,offset retData
	.SQL_DESC_CASE_SENSITIVE
	invoke ListviewAddItem, [hLv], IMG_NUMERIC, offset ValueLabel10, offset retData, 0

	// SQL_DESC_UNSIGNED
	// Boolean value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_UNSIGNED,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
		jmp >.SQL_DESC_UNSIGNED
	:
	invoke STRING2BOOL,offset retData
	.SQL_DESC_UNSIGNED
	invoke ListviewAddItem, [hLv], IMG_NUMERIC, offset ValueLabel11, offset retData, 0

	// SQL_DESC_UPDATABLE
	// Boolean value
	invoke ODBCGetColumnInfo,offset szTable,[dwColumn],SQL_DESC_UPDATABLE,offset szDbFile, offset retData
	test eax,eax
	jns >
		invoke lstrcpy,offset retData,offset szError
		jmp >.SQL_DESC_UPDATABLE
	:
	invoke STRING2BOOL,offset retData
	.SQL_DESC_UPDATABLE
	invoke ListviewAddItem, [hLv], IMG_NUMERIC, offset ValueLabel12, offset retData, 0

	call ODBCDisconnect
	.EXIT
	RET
ENDF

STRING2BOOL FRAME pString
	uses edi

	mov eax,[pString]
	mov edi,[eax]
	invoke lstrcpy,[pString],"TRUE"
	and edi,0ffh
	cmp edi, "1"
	je >
	invoke lstrcpy,[pString],"FALSE"
	:
	RET
ENDF

ListviewAddItem FRAME hLv, Image, pDesc, pData, lParam
	LOCAL lvi		:LVITEM
	LOCAL ItemID	:D

	invoke SendMessage,[hLv],LVM_GETITEMCOUNT,0,0
	mov [ItemID],eax

	mov D[lvi.mask],LVIF_PARAM + LVIF_TEXT + LVIF_IMAGE
	mov D[lvi.iSubItem],0
	mov eax,[pDesc]
	mov [lvi.pszText],eax

	mov eax,[Image]
	mov [lvi.iImage],eax
	mov eax,[lParam]
	mov D[lvi.lParam],eax
	mov eax,[ItemID]
	mov [lvi.iItem],eax
	invoke SendMessage,[hLv],LVM_INSERTITEM,0,OFFSET lvi
	mov [ItemID],eax

	mov eax,[pData]
	mov [lvi.pszText],eax
	mov eax,[ItemID]
	mov [lvi.iItem],eax
	mov D[lvi.iSubItem],1
	mov D[lvi.mask],LVIF_TEXT
	invoke SendMessage,[hLv],LVM_SETITEM,0,OFFSET lvi

	RET
ENDF

TrvAddItem FRAME hTrv,hPar,lpString,nImg, bChildren, iColumn
	LOCAL	tvins			:TVINSERTSTRUCT

	invoke HeapAlloc,[hHeap],HEAP_ZERO_MEMORY,SIZEOF COLUMNINFOSTRUCT
	mov D[tvins.itemex.lParam],eax

	mov edx,[hPar]
	mov [tvins.hParent],edx
	// Save the parent handle in lParam so it's easy to find
	mov [eax+COLUMNINFOSTRUCT.hParent], edx
	mov edx, [iColumn]
	mov [eax+COLUMNINFOSTRUCT.iColumn], edx

	mov D[tvins.hInsertAfter],0
	mov D[tvins.itemex.mask],TVIF_TEXT + TVIF_PARAM + TVIF_IMAGE + TVIF_SELECTEDIMAGE + TVIF_CHILDREN
	mov eax,[lpString]
	mov [tvins.itemex.pszText],eax

	mov eax,[bChildren]
	mov D[tvins.itemex.cChildren],eax

	mov eax,[nImg]
	mov D[tvins.itemex.iImage],eax
	mov D[tvins.itemex.iSelectedImage],eax
	invoke SendMessage,[hTrv],TVM_INSERTITEM,0,OFFSET tvins

    ret

ENDF
