ODBCDATA STRUCT
	cbStruct DD SIZEOF ODBCDATA
	hConn DD
	hEnv DD
ENDS

TABLES STRUCT
	Catalog 	DB 128 DUP (?) 	;The name of the catalog containing TABLE_SCHEM. This column contains a null value.
	cbCatalog	DD ?
	Schema 		DB 128 DUP (?)	;The name of the schema containing TABLE_NAME.
	cbSchema	DD ?
	Name 		DB 128 DUP (?)	;The name of the table, or view, or alias, or synonym.
	cbName 		DD ?
	Type 		DB 128 DUP (?)	;Identifies the type given by the name in the TABLE_NAME column. It can have the string values 'TABLE', 'VIEW', 'INOPERATIVE VIEW', 'SYSTEM TABLE', 'ALIAS', or 'SYNONYM'.
	cbType 		DD ?
	Remarks 	DB 254 DUP (?)	;Contains the descriptive information about the table.
	cbRemarks	DD ?
ENDS

COLUMNS STRUCT
	Catalog 		DB 128 DUP (?)
	cbCatalog		DD ?
	Schema 			DB 128 DUP (?)
	cbSchema		DD ?
	TableName 		DB 128 DUP (?)
	cbTableName		DD ?
	ColumnName 		DB 128 DUP (?)
	cbColumnName 	DD ?
	DataType		DW ?
	cbDataType	 	DD ?
	TypeName 		DB 128 DUP (?)
	cbTypeName	 	DD ?
	ColumnSize		DD ?
	cbColumnSize	DD ?
	BufferLength	DD ?
	cbBufferLength	DD ?
	DecimalDigits	DD ?
	cbDecimalDigits	DD ?
	PrecRadix		DD ?
	cbPrecRadix		DD ?
	Nullable		DD ?
	cbNullable		DD ?
	Remarks 		DB 254 DUP (?)
	cbRemarks		DD ?
	ColumnDef		DB 128 DUP (?)
	cbColumnDef		DD ?
	SQL_DATA_TYPE		DW ?
	cbSQL_DATA_TYPE		DD ?
	SQL_DATETIME_SUB	DW ?
	cbSQL_DATETIME_SUB	DD ?
	CHAR_OCTET_LENGTH	DD ?
	cbCHAR_OCTET_LENGTH	DD ?
	ORDINAL_POSITION	DD ?
	cbORDINAL_POSITION	DD ?
	IS_NULLABLE			DB 128 DUP (?)
	cbIS_NULLABLE		DD ?
ENDS

COLUMNINFOSTRUCT STRUCT
	hParent	DD
	iColumn	DD
ENDS

IMG_ALIAS equ 0
IMG_TABLE equ 1
IMG_SYNONYM equ 2
IMG_SYSTEM_TABLE equ 3
IMG_VIEW equ 4
IMG_GLOBAL_TEMPORARY equ 5
IMG_LOCAL_TEMPORARY equ 6
IMG_SYSTEM_VIEW equ 7

CONST SECTION
	ALIGN 4
	szFilterStr			DB	"MSJet database",0,"*.mdb",0,0
	ALIGN 4
	szDefExt			DB	"mdb",0
	ALIGN 4
	szOpenCaption		DB	"Choose a database",0
	ALIGN 4
	szConnect 			DB "DRIVER={Microsoft Access Driver (*.mdb)};DBQ=",0

DATA SECTION
	szDbFile			DB	MAX_PATH DUP (?)
	szSQLQuery			DB	256 dup (?)
	odbcdata			ODBCDATA <>

CODE SECTION

// Disconnect from database
ODBCDisconnect:

	cmp D[odbcdata.hConn], 0
	je >
		invoke SQLDisconnect,[odbcdata.hConn]
		invoke SQLFreeHandle,SQL_HANDLE_DBC,[odbcdata.hConn]
		invoke SQLFreeHandle,SQL_HANDLE_ENV,[odbcdata.hEnv]
		mov	D[odbcdata.hConn],0
		mov	D[odbcdata.hEnv],0
	:
	ret

OpenDatabase FRAME hwnd, pFilebuffer
	LOCAL ofn:OPENFILENAME
	invoke RtlZeroMemory,offset ofn,SIZEOF OPENFILENAME

	mov D[ofn.lStructSize],SIZEOF OPENFILENAME
	mov eax,[hwnd]
	mov [ofn.hwndOwner],eax
	mov eax,[hInstance]
	mov D[ofn.hInstance],NULL
	mov D[ofn.lpstrFilter],OFFSET szFilterStr
	mov D[ofn.lpstrCustomFilter],NULL
	mov D[ofn.nMaxCustFilter],NULL
	mov D[ofn.nFilterIndex],1
	mov eax,[pFilebuffer]
	mov D[eax],0
	mov [ofn.lpstrFile],eax
	mov D[ofn.nMaxFile],MAX_PATH
	mov D[ofn.lpstrFileTitle],NULL
	mov D[ofn.nMaxFileTitle],NULL
	mov D[ofn.lpstrInitialDir],NULL
	mov [ofn.lpstrTitle],OFFSET szOpenCaption
	mov D[ofn.Flags],OFN_NONETWORKBUTTON + OFN_EXPLORER + OFN_FILEMUSTEXIST
	mov D[ofn.lpfnHook],NULL
	mov D[ofn.lpTemplateName],NULL
	mov D[ofn.lpstrDefExt],OFFSET szDefExt
	mov D[ofn.lCustData],NULL

	invoke GetOpenFileName,OFFSET ofn
	or eax,eax
	jz >>.ERROR
		xor eax,eax
	RET

	.ERROR
	xor eax,eax
	dec eax
	RET
ENDF

// Connect to database
ODBCConnect FRAME lpFilename
	uses esi, edi
	LOCAL	lnCon:D

	invoke SQLAllocHandle,SQL_HANDLE_ENV,SQL_NULL_HANDLE,offset odbcdata.hEnv
	cmp ax,SQL_SUCCESS
	je >
	cmp ax,SQL_SUCCESS_WITH_INFO
	je >
	ret
	:

	invoke SQLSetEnvAttr,[odbcdata.hEnv],SQL_ATTR_ODBC_VERSION,SQL_OV_ODBC3,0
	cmp ax,SQL_SUCCESS
	je >
	cmp ax,SQL_SUCCESS_WITH_INFO
	jne >>.ERROR
	:

	invoke SQLAllocHandle,SQL_HANDLE_DBC,[odbcdata.hEnv],offset odbcdata.hConn
	cmp ax,SQL_SUCCESS
	je >
	cmp ax,SQL_SUCCESS_WITH_INFO
	jne >>.ERROR
	:
	invoke HeapAlloc,[hHeap],HEAP_ZERO_MEMORY,1024
	mov esi,eax
	invoke lstrcpy,esi,offset szConnect
	invoke lstrcat,esi,[lpFilename]
	invoke HeapAlloc,[hHeap],HEAP_ZERO_MEMORY,1024
	mov edi,eax
	invoke lstrlen,esi
	mov edx,eax
	invoke SQLDriverConnect,[odbcdata.hConn],0,esi,edx,edi,1024,addr lnCon, SQL_DRIVER_COMPLETE
	cmp ax,SQL_SUCCESS
	je >
	cmp ax,SQL_SUCCESS_WITH_INFO
	jne >.ERROR1
	:
	invoke HeapFree,[hHeap],0,esi
	invoke HeapFree,[hHeap],0,edi
	xor eax,eax
	ret

	.ERROR1
	invoke MessageBox,0,"Error connecting to data base",0,0
	invoke SQLFreeHandle,SQL_HANDLE_DBC,[odbcdata.hConn]
	invoke SQLFreeHandle,SQL_HANDLE_ENV,[odbcdata.hEnv]
	invoke HeapFree,[hHeap],0,esi
	invoke HeapFree,[hHeap],0,edi
	mov	D[odbcdata.hConn],0
	mov	D[odbcdata.hEnv],0
	xor eax,eax
	dec eax
	ret

	.ERROR
	invoke MessageBox,0,"Error connecting to data base",0,0
	invoke SQLFreeHandle,SQL_HANDLE_ENV,[odbcdata.hEnv]
	mov D[odbcdata.hEnv],0
	xor eax,eax
	dec eax
	ret

ENDF

// Get a statement handle and set some attributes
ODBCGetStatementHandle FRAME
	LOCAL	hStmt:D

	invoke SQLAllocHandle, SQL_HANDLE_STMT, [odbcdata.hConn],addr hStmt
	cmp ax, SQL_SUCCESS
	je >
	cmp ax, SQL_SUCCESS_WITH_INFO
	jne >.ERROR
	:
		invoke SQLSetStmtAttr,[hStmt], SQL_ATTR_CONCURRENCY, SQL_CONCUR_ROWVER, 0
		invoke SQLSetStmtAttr,[hStmt], SQL_ATTR_CURSOR_TYPE, SQL_CURSOR_KEYSET_DRIVEN, 0
		mov eax,[hStmt]
		ret

	.ERROR
		invoke MessageBox, NULL, 'Unable to allocate statement handle.', 0, 0
		xor eax,eax
ENDF

ODBCBindTableCols FRAME hStmt, lpData
	uses ebx

	mov	ebx,[lpData]

	lea eax,[ebx+TABLES.Catalog]
	lea ecx,[ebx+TABLES.cbCatalog]
	invoke SQLBindCol,[hStmt], 1, SQL_C_CHAR, eax, 128, ecx

	lea eax,[ebx+TABLES.Schema]
	lea ecx,[ebx+TABLES.cbSchema]
	invoke SQLBindCol,[hStmt], 2, SQL_C_CHAR, eax, 128, ecx

	lea eax,[ebx+TABLES.Name]
	lea ecx,[ebx+TABLES.cbName]
	invoke SQLBindCol,[hStmt], 3, SQL_C_CHAR, eax, 128, ecx

	lea eax,[ebx+TABLES.Type]
	lea ecx,[ebx+TABLES.cbType]
	invoke SQLBindCol,[hStmt], 4, SQL_C_CHAR, eax, 128, ecx

	lea eax,[ebx+TABLES.Remarks]
	lea ecx,[ebx+TABLES.cbRemarks]
	invoke SQLBindCol,[hStmt], 5, SQL_C_CHAR, eax, 254, ecx
	ret

ENDF

ODBCGetTableList FRAME pszFilename
	uses ebx, edi, esi
	LOCAL	hStmt:D
	LOCAL	icon:D

	mov D[icon],0

	invoke ODBCGetStatementHandle
	mov [hStmt],eax

	invoke GlobalAlloc,GMEM_FIXED,SIZEOF TABLES
	mov ebx,eax

	invoke ODBCBindTableCols, [hStmt], ebx

	invoke SQLTables,[hStmt], NULL, 0, NULL, 0, NULL, 0, NULL,0

	.READ_RECORD
		mov	B[ebx+TABLES.Catalog],0
		mov	B[ebx+TABLES.Schema],0
		mov	B[ebx+TABLES.Name],0
		mov	B[ebx+TABLES.Type],0
		mov	B[ebx+TABLES.Remarks],0

		invoke SQLFetch,[hStmt]
		cmp ax,SQL_SUCCESS
		je >
		cmp ax,SQL_SUCCESS_WITH_INFO
		jne >>.END_READ
		:
		cmp ax,SQL_NO_DATA
		je >>.END_READ
			lea esi,[ebx+TABLES.Type]
		;	invoke lstrcmp,esi,"SYSTEM TABLE"
		;	test eax,eax
		;	jz <.READ_RECORD
			// Determine the right icon
			invoke lstrcmp,esi,"ALIAS"
			test eax,eax
			jnz >
				mov D[icon], IMG_ALIAS
				jmp >>.ADDRECORD
			:
			invoke lstrcmp,esi,"TABLE"
			test eax,eax
			jnz >
				mov D[icon], IMG_TABLE
				jmp >>.ADDRECORD
			:
			invoke lstrcmp,esi,"SYNONYM"
			test eax,eax
			jnz >
				mov D[icon], IMG_SYNONYM
				jmp >>.ADDRECORD
			:
			invoke lstrcmp,esi,"SYSTEM TABLE"
			test eax,eax
			jnz >
				mov D[icon], IMG_SYSTEM_TABLE
				jmp >>.ADDRECORD
			:
			invoke lstrcmp,esi,"VIEW"
			test eax,eax
			jnz >
				mov D[icon], IMG_VIEW
				jmp >>.ADDRECORD
			:
			invoke lstrcmp,esi,"GLOBAL TEMPORARY"
			test eax,eax
			jnz >
				mov D[icon], IMG_GLOBAL_TEMPORARY
				jmp >>.ADDRECORD
			:
			invoke lstrcmp,esi,"LOCAL TEMPORARY"
			test eax,eax
			jnz >
				mov D[icon], IMG_LOCAL_TEMPORARY
				jmp >>.ADDRECORD
			:
			invoke lstrcmp,esi,"SYSTEM VIEW"
			test eax,eax
			jnz >
				mov D[icon], IMG_SYSTEM_VIEW
			:
			.ADDRECORD
			lea edi,[ebx+TABLES.Name]
			// Add the node
			invoke TrvAddItem,[hTreeview],0,edi,[icon],TRUE,0

			// Get the table's column names
			invoke ODBCGetColumnList,edi,eax,[pszFilename]

			jmp	<.READ_RECORD
		.END_READ
		invoke SQLFreeHandle,SQL_HANDLE_STMT,[hStmt]
	.EXIT
	
	invoke GlobalFree, ebx
	RET
ENDF

ODBCGetTableInfo FRAME pszTablename, pszFilename
	uses ebx, edi
	LOCAL	hStmt:D

	invoke ODBCGetStatementHandle
	mov [hStmt],eax

	invoke GlobalAlloc,GMEM_FIXED,SIZEOF TABLES
	mov ebx,eax

	invoke ODBCBindTableCols, [hStmt], ebx

	invoke SQLTables,[hStmt], NULL, 0, NULL, 0, NULL, 0, NULL,0

	.READ_RECORD
		mov	B[ebx+TABLES.Catalog],0
		mov	B[ebx+TABLES.Schema],0
		mov	B[ebx+TABLES.Name],0
		mov	B[ebx+TABLES.Type],0
		mov	B[ebx+TABLES.Remarks],0

		invoke SQLFetch,[hStmt]
		cmp ax,SQL_SUCCESS
		je >
		cmp ax,SQL_SUCCESS_WITH_INFO
		jne >.END_READ
		:
		cmp ax,SQL_NO_DATA
		je >.END_READ
			lea eax,[ebx+TABLES.Type]
			invoke lstrcmp,eax,"SYSTEM TABLE"
			test eax,eax
			jz <.READ_RECORD
			lea edi,[ebx+TABLES.Name]
			// Add the node
			invoke TrvAddItem,[hTreeview],0,edi,0,TRUE,0

			// Get the table's column names
			invoke ODBCGetColumnList,edi,eax,[pszFilename]

			jmp	<.READ_RECORD
		.END_READ
		invoke SQLFreeHandle,SQL_HANDLE_STMT,[hStmt]
	.EXIT
	
	invoke GlobalFree, ebx
	RET
ENDF

ODBCGetColumnList FRAME pszTableName, hParent, pszFilename
	uses ebx, edi
	LOCAL	hStmt:D

	xor edi,edi

	invoke ODBCGetStatementHandle
	mov [hStmt],eax

	invoke GlobalAlloc,GMEM_FIXED,SIZEOF COLUMNS
	mov ebx,eax

	invoke ODBCBindTableCols, [hStmt], ebx

	invoke lstrlen,[pszTableName]
	invoke SQLColumns,[hStmt], NULL, 0, NULL, 0, [pszTableName], eax, NULL,0

	.READ_RECORD
		inc edi
		mov	B[ebx+COLUMNS.Catalog],0
		mov	B[ebx+COLUMNS.Schema],0
		mov	B[ebx+COLUMNS.TableName],0
		mov	B[ebx+COLUMNS.ColumnName],0
		mov	B[ebx+COLUMNS.DataType],0
		mov	B[ebx+COLUMNS.TypeName],0
		mov	D[ebx+COLUMNS.ColumnSize],0
		mov	B[ebx+COLUMNS.BufferLength],0
		mov	B[ebx+COLUMNS.DecimalDigits],0
		mov	B[ebx+COLUMNS.PrecRadix],0
		mov	B[ebx+COLUMNS.Nullable],0
		mov	B[ebx+COLUMNS.Remarks],0
		mov	B[ebx+COLUMNS.ColumnDef],0
		mov	B[ebx+COLUMNS.SQL_DATA_TYPE],0
		mov	B[ebx+COLUMNS.SQL_DATETIME_SUB],0
		mov	B[ebx+COLUMNS.CHAR_OCTET_LENGTH],0
		mov	B[ebx+COLUMNS.ORDINAL_POSITION],0
		mov	B[ebx+COLUMNS.IS_NULLABLE],0

		invoke SQLFetch,[hStmt]
		cmp ax,SQL_SUCCESS
		je >
		cmp ax,SQL_SUCCESS_WITH_INFO
		jne >.END_READ
		:
		cmp ax,SQL_NO_DATA
		je >.END_READ
			lea eax,[ebx+COLUMNS.ColumnName]
			invoke TrvAddItem,[hTreeview],[hParent],eax,8,0,edi
			jmp	<.READ_RECORD
		.END_READ
		invoke SQLFreeHandle,SQL_HANDLE_STMT,[hStmt]
	.EXIT
	invoke GlobalFree, ebx
	
	RET
ENDF

ODBCGetColumnInfo FRAME pszTableName, ColumnNumber, FieldIdentifier, pszFilename, pRet
	uses ebx, edi
	LOCAL hStmt		:D
	LOCAL fConnect	:D
	LOCAL fNumeric	:D
	LOCAL szCharacterAttribute[1024]:B
	LOCAL StringLength	:D
	LOCAL NumericAttribute	:D

	mov D[fConnect],0
	mov D[fNumeric],-1

	xor edi,edi

	invoke GlobalAlloc,GMEM_FIXED,260
	mov ebx,eax

	// Build our query
	invoke wsprintf,offset szSQLQuery,"SELECT * FROM %s",[pszTableName]
	add esp,12

	invoke ODBCGetStatementHandle
	mov [hStmt],eax

	// Since it can generate an error to bind a higher-numbered column than there are columns
	// Just bind to the bookmark column for our fetch.
	lea eax,[ebx+255]
	invoke SQLBindCol,[hStmt], 0, SQL_C_CHAR, ebx, 254, eax

	// Execute and fetch a simple query in order to get a result set
	invoke SQLExecDirect, [hStmt], offset szSQLQuery, SQL_NTS
	invoke SQLFetch,[hStmt]

	// Zero our buffers
	mov D[szCharacterAttribute],0
	mov D[StringLength], 0
	mov D[NumericAttribute], 0

	// Get the attribute we are looking for
	invoke SQLColAttribute,[hStmt], [ColumnNumber], [FieldIdentifier], offset szCharacterAttribute, 1024, offset StringLength, offset NumericAttribute
	cmp B[szCharacterAttribute],0
	je >>.ISNUMERIC

	lea eax, szCharacterAttribute
	invoke lstrcpy,[pRet],eax
	mov D[fNumeric], FALSE
	jmp >>.FREEODBC

	.ISNUMERIC
	invoke wsprintf,[pRet], "%u", [NumericAttribute]
	add esp,12
	mov D[fNumeric], TRUE

	.FREEODBC
	invoke SQLFreeHandle,SQL_HANDLE_STMT,[hStmt]

	.EXIT
	invoke GlobalFree, ebx
	mov eax,[fNumeric]
	RET
ENDF
